import re

from urllib.request import unquote, urlopen
from urllib.parse import urlparse
from remotecv.timing import get_time, get_interval
from remotecv.utils import context


def load_sync(path):
    start_time = get_time()
    if not re.match(r"^https?", path):
        path = f"http://{path}"
    path = unquote(path)
    netloc = urlparse(path).netloc.replace(".", "_")
    response = urlopen(path)
    code = response.code
    result = response.read()

    context.metrics.incr("worker.original_image.response_bytes", len(result))
    context.metrics.timing(
        f"worker.original_image.fetch.{code}.{netloc}",
        get_interval(start_time, get_time()),
    )
    context.metrics.incr(
        f"worker.original_image.fetch.{code}.{netloc}",
    )
    context.metrics.incr(f"worker.original_image.status.{code}")
    context.metrics.incr(f"worker.original_image.status.{code}.{netloc}")

    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

from remotecv.detectors.complete_detector import CompleteDetector
from remotecv.detectors.face_detector import FaceDetector
from remotecv.detectors.feature_detector import FeatureDetector
from remotecv.detectors.glasses_detector import GlassesDetector
from remotecv.detectors.profile_detector import ProfileDetector
from remotecv.image import Image
from remotecv.timing import get_time, get_interval
from remotecv.utils import context


class ImageProcessor:
    def __init__(self):
        self.detectors = {
            "all": CompleteDetector(),
            "face": FaceDetector(),
            "feature": FeatureDetector(),
            "glass": GlassesDetector(),
            "profile": ProfileDetector(),
        }

    def detect(self, detector, image_data):
        image = Image.create_from_buffer(image_data)
        if image is None:
            return []

        return self.run_detections(image, detector.split("+"))

    def run_detections(self, image, detectors):
        result = []
        for detector in detectors:
            try:
                start_time = get_time()
                points = self.detectors[detector].detect(image)
                context.metrics.timing(
                    f"worker.{detector}.time",
                    get_interval(start_time, get_time()),
                )
                if points:
                    context.metrics.incr(f"worker.{detector}.detected")

                result = result + points
            except KeyError as key_error:
                raise AttributeError("Detector unavailable") from key_error

        return result



from http.server import BaseHTTPRequestHandler


class HealthCheckHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)
        self.send_header("Cache-Control", "no-cache")
        self.end_headers()
        self.wfile.write(b"WORKING")



# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2022, globo.com <thumbor@g.globo>

from remotecv.metrics import BaseMetrics
from remotecv.utils import logger


class Metrics(BaseMetrics):
    def incr(self, metricname, value=1):
        logger.debug("METRICS: inc: %s:%d", metricname, value)

    def timing(self, metricname, value):
        logger.debug("METRICS: timing: %s:%d", metricname, value)



# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2022, globo.com <thumbor@g.globo>


class BaseMetrics:
    def __init__(self, config):
        self.config = config

    def initialize(self):
        pass

    def incr(self, metricname, value=1):
        raise NotImplementedError()

    def timing(self, metricname, value):
        raise NotImplementedError()



# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2022, globo.com <thumbor@g.globo>

import time


def get_time():
    return time.perf_counter_ns()


def get_interval(start, end):
    return nano_to_ms(end - start)


def nano_to_ms(ns_time):
    return ns_time / 1e6



# -*- coding: utf-8 -*-

# thumbor imaging service
# https://github.com/thumbor/thumbor/wiki

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2022, globo.com <thumbor@g.globo>

from importlib import import_module


class Importer:
    @classmethod
    def import_class(cls, class_name, conf_value):
        module = import_module(conf_value)
        return getattr(module, class_name)
